from rocolib.api.components.Component import Component
from rocolib.api.Function import Function

#ESP3 STACK WITH PWM SERVO FEATHERWING
c = Component()

c.addSubcomponent("holder", "SimpleRectBeam")

#Dimensions of the ESP32 Stack
c.addParameter("brains", "esp32stack", paramType="dimension")
c.addConstraint(("holder", "length"), "brains", "getDim(x, 'length')")
c.addConstraint(("holder", "depth"), "brains", "getDim(x, 'width')")
c.addConstraint(("holder", "width"), "brains", "getDim(x, 'height')")

#This parameter will change when the dimensions of the car or vehicle is changed
#so that it would not affect the ESP32 stack holder
c.addParameter("side", 10, parameterType="length")

#Adding splitedge to attach to the sides of the ESP32 stack holder
for i in range(4):
    c.addSubcomponent("split%d" %i, "SplitEdge")
    c.addConstraint(("split%d" %i, "toplength"), "brains", "(getDim(x, 'height'),)")
    if i == 0 or i == 1:
        c.addConstraint(("split%d" %i, "botlength"), "brains", "(8, getDim(x, 'height')-8)")
    else:
        c.addConstraint(("split%d"%i, "botlength"), "brains", "(getDim(x, 'height')-8, 8)")

c.addConnection(("holder", "topedge0"), ("split0", "topedge0"))
c.addConnection(("holder", "botedge2"), ("split1", "topedge0"))
c.addConnection(("holder", "topedge2"), ("split2", "topedge0"))
c.addConnection(("holder", "botedge0"), ("split3", "topedge0"))

#These are the sides that will change when the length of the car changes
c.addSubcomponent("side0", "SimpleRectBeam")
c.addSubcomponent("side1", "SimpleRectBeam")

for i in range(2):
    c.addConstraint(("side%d" %i, "length"), "side")
    c.addConstraint(("side%d" %i, "depth"), "brains", "getDim(x, 'width')")
    c.addConstraint(("side%d" %i, "width"), "brains", "getDim(x, 'height')-8")

c.addConnection(("split0", "botedge1"), ("side0", "botedge2"))
c.addConnection(("split2", "botedge0"), ("side0", "botedge0"), tabWidth=7)
c.addConnection(("split1", "botedge1"), ("side1", "botedge2"))
c.addConnection(("split3", "botedge0"), ("side1", "botedge0"), tabWidth=7)

#OLED at the top
c.addParameter("dx1", 40, parameterType="length")
c.addParameter("dy1", 15, parameterType="length")
c.addSubcomponent("led", "Cutout")

c.addConstraint(("led", "dx"), "dx1")
c.addConstraint(("led", "dy"), "dy1")

c.addConnection(("holder", "face1"),
                   ("led", "decoration"),
                   mode="hole", rotate=True, offset=Function(params=("brains"), fnstring="(0, 3)"))

#Holes for servo and powersupply
for i in range(2):
    c.addSubcomponent("servohole%d" %i, "Cutout")
    c.addConstraint(("servohole%d" %i, "dy"), "dy1", "x+2")
    c.addConstraint(("servohole%d" %i, "dx"), "side", "7")

c.addSubcomponent("powerhole", "Cutout")
c.addConstraint(("powerhole", "dy"), "dy1", "x+4")
c.addConstraint(("powerhole", "dx"), "side", "13")

c.addConnection(("side0", "face2"),
                   ("servohole0", "decoration"),
                   mode="hole", rotate=True, offset=Function(params=("side"), fnstring="(1, 0)"))

c.addConnection(("side1", "face2"),
                   ("servohole1", "decoration"),
                   mode="hole", rotate=True, offset=Function(params=("side"), fnstring="(1, 0)"))

c.addConnection(("holder", "face2"),
                   ("powerhole", "decoration"),
                   mode="hole", rotate=True, offset=Function(params=("brains"), fnstring="(2, 12)"))

c.inheritAllInterfaces("side0")
c.inheritAllInterfaces("side1")
c.inheritAllInterfaces("holder", prefix=None)

c.toLibrary("ESP32Stack")